import json
import tempfile
import os
from googleapiclient.discovery import build
from googleapiclient.http import MediaFileUpload
from google.oauth2 import service_account
from shipyard_templates import CloudStorage, ExitCodeException
from typing import Optional, Dict, List, Any, Union
from googleapiclient.http import MediaIoBaseDownload
from shipyard_googledrive import drive_utils


class GoogleDriveClient(CloudStorage):
    SCOPES = ["https://www.googleapis.com/auth/drive"]
    EXIT_CODE_DRIVE_ACCESS_ERROR = 209

    def __init__(
        self,
        service_account: str,
        shared_drive_name: Optional[str] = None,
    ) -> None:
        self.service_account = service_account
        self.shared_drive_name = shared_drive_name
        self.service = self.connect()
        self.drive_id = None
        self.folder_id = None
        self.folder_name = None
        super().__init__(
            service_account=self.service_account,
            service=self.service,
            shared_drive_name=self.shared_drive_name,
            drive_id=self.drive_id,
            folder_id=self.folder_id,
            folder_name=self.folder_name,
        )

    @property
    def email(self):
        json_creds = json.loads(self.service_account)
        return json_creds["client_email"]

    def _set_env_vars(self):
        try:
            fd, path = tempfile.mkstemp()
            with os.fdopen(fd, "w") as tmp:
                tmp.write(self.service_account)
            os.environ["GOOGLE_APPLICATION_CREDENTIALS"] = path
            return path
        except Exception as e:
            os.environ["GOOGLE_APPLICATION_CREDENTIALS"] = self.service_account

    def _store_json_credentials(self):
        with open("credentials.json", "w") as f:
            json.dump(self.service_account, f)

    def connect(self):
        tmp_path = self._set_env_vars()
        credentials = service_account.Credentials.from_service_account_file(
            tmp_path, scopes=self.SCOPES
        )
        service = build("drive", "v3", credentials=credentials)
        return service

    def upload(
        self,
        file_path: str,
        drive_folder: Optional[str] = None,
        drive_file_name: Optional[str] = None,
        drive: Optional[str] = None,
    ):
        """Uploads a file to a shared Google Drive

        Args:
            file_path: The path of the file to load
            drive_file_name: The name that the uploaded file in drive will have
            drive_folder: The ID of the folder in Google drive
            drive: The name or ID of the shared drive
        """

        # NOTE: check to see that the folder is shared with the service account
        if drive:
            self.shared_drive_name = drive
            self.drive_id = drive_utils.get_drive_id(
                service=self.service, drive_id=self.shared_drive_name
            )
            # if the drive is provided, but not found, then the service account doesn't have access and needs it to be shared
            if not self.drive_id:
                raise ExitCodeException(
                    f"{self.email} does not have access to the following drive: {self.shared_drive_name}. Please visit the authorization guide to see how to share the Drive to the service account",
                    self.EXIT_CODE_DRIVE_ACCESS_ERROR,
                )

        try:
            if drive_folder:
                self.folder_name = drive_folder
                self.folder_id = drive_utils.get_folder_id(
                    service=self.service,
                    folder_identifier=self.folder_name,
                    drive_id=self.drive_id,
                )
                if not self.folder_id and not drive_utils.is_folder_id(
                    self.folder_name
                ):
                    folder_results = drive_utils.create_remote_folder(
                        folder_name=self.folder_name,
                        service=self.service,
                        drive_id=self.drive_id,
                    )
                    self.folder_id = folder_results
                # else:
                #     if not drive_utils.is_folder_shared(service_account_email=self.email,folder_id = self.folder_id, drive_service= self.service):
                #         raise ExitCodeException(f'Error: The folder {self.folder_name} exists in Google Drive but has not been shared with the service account associated with {self.email}. Share the folder, then retry the upload', self.EXIT_CODE_FOLDER_ACCESS_ERROR)

            # use the base name of the file if not provided
            if not drive_file_name:
                drive_file_name = os.path.basename(file_path)
            file_metadata = {"name": drive_file_name, "parents": []}
            # if the folder exists, check to see if it has been shared with the drive correctly, if not then throw exception
            if self.folder_id:
                # FIXME: This throws an exception every time, needs to be fixed
                # if drive_utils.is_folder_shared(
                #     service_account_email=self.email,
                #     folder_id=self.folder_id,
                #     drive_service=self.service,
                # ):
                file_metadata["parents"].append(self.folder_id)
                # raise ExitCodeException(
                #     f"Aborting upload. The folder with ID {self.folder_id} exists but has not been shared with the service account {self.email}. Please share that folder with the service account then retry uploading.",
                #     exit_code=self.EXIT_CODE_FOLDER_ACCESS_ERROR,
                # )
            elif self.drive_id:
                self.folder_id = self.drive_id
                file_metadata["parents"].append(self.folder_id)
            else:
                self.folder_id = "root"
                file_metadata["parents"].append(self.folder_id)

            # check to see if the file exists or not
            update = False
            if drive_utils.does_file_exist(
                logger=self.logger,
                parent_folder_id=self.folder_id,
                file_name=drive_file_name,
                service=self.service,
                drive_id=self.drive_id,
            ):
                parents = file_metadata.pop("parents")
                update = True
                if parents != []:
                    self.folder_id = parents[0]  # update the folder ID

            media = MediaFileUpload(file_path, resumable=True)
            if update:
                file_id = drive_utils.get_file_id(
                    file_name=drive_file_name,
                    service=self.service,
                    drive_id=self.drive_id,
                    folder_id=self.folder_id,  # NOTE: This was added after tests. Need to retest
                )
                upload_file = (
                    self.service.files()
                    .update(
                        fileId=file_id,
                        body=file_metadata,
                        media_body=media,
                        supportsAllDrives=True,
                        fields=("id"),
                        addParents=self.folder_id,
                    )
                    .execute()
                )
                self.logger.info(f"Updated file {file_id}")

            else:
                upload_file = (
                    self.service.files()
                    .create(
                        body=file_metadata,
                        media_body=media,
                        fields=("id"),
                        supportsAllDrives=True,
                    )
                    .execute()
                )
                self.logger.info(f"Newly created file is {upload_file.get('id')}")
        except FileNotFoundError as fe:
            raise ExitCodeException(
                message=str(fe), exit_code=self.EXIT_CODE_FILE_NOT_FOUND
            )
        except ExitCodeException as ec:
            raise ExitCodeException(message=ec.message, exit_code=ec.exit_code)
        except Exception as e:
            raise ExitCodeException(
                message=f"Error in uploading file to google drive: {str(e)}",
                exit_code=self.EXIT_CODE_UPLOAD_ERROR,
            )

    def move(self):
        pass

    def remove(self):
        pass

    def download(
        self,
        file_id: str,
        drive_file_name: str,
        drive_folder: Optional[str] = None,
        drive: Optional[str] = None,
        destination_path: Optional[str] = None,
        destination_file_name: Optional[str] = None,
    ):
        """Downloads a file from Google Drive locally

        Args:
            file_id: The ID of the file to download
            drive_file_name: The name of the file to download
            drive_folder: The optional name of the folder or the ID of folder. If not provided, then it will look for the file within the root directory of the drive
            drive: The optional name or ID of the shared drive
            destination_path: The optional path to download the file to, if not provided, then the file will be downloaded to the current working directory
            destination_file_name: The optional name of the downloaded file to have. If not provided, then the file will have the same name as it did in Google Drive
        """
        if drive:
            self.drive_id = drive_utils.get_drive_id(
                drive_id=drive, service=self.service
            )

        if drive_folder:
            try:
                self.folder_id = drive_utils.get_folder_id(
                    folder_identifier=drive_folder,
                    service=self.service,
                    drive_id=self.drive_id,
                )
            except ExitCodeException as ec:
                raise ExitCodeException(ec.message, ec.exit_code)

        if destination_path:
            # if exists, then skip
            if not os.path.exists(destination_path):
                os.mkdir(destination_path)
        else:
            destination_path = os.getcwd()

        if destination_file_name:
            destination_path = os.path.join(destination_path, destination_file_name)
        else:
            destination_path = os.path.join(destination_path, drive_file_name)

        try:
            request = self.service.files().get_media(fileId=file_id)
            fh = open(destination_path, "wb")
            downloader = MediaIoBaseDownload(fh, request)
            complete = False
            while not complete:
                status, complete = downloader.next_chunk()
        except Exception as e:
            raise ExitCodeException(
                message=str(e), exit_code=self.EXIT_CODE_DOWNLOAD_ERROR
            )
        else:
            return



import os
import sys
from shipyard_blueprints import GoogleDriveClient


def main():
    drive = GoogleDriveClient(os.getenv("GOOGLE_APPLICATION_CREDENTIALS"))
    try:
        conn = drive.connect()
        drive.logger.info("Successfully connected to Google Drive")
        sys.exit(0)
    except Exception as e:
        drive.logger.error(
            "Could not connect to Google Drive with the service account provided"
        )
        sys.exit(1)


if __name__ == "__main__":
    main()



import os
import sys
import argparse

from shipyard_templates import ExitCodeException
from shipyard_googledrive import GoogleDriveClient, drive_utils
from shipyard_bp_utils import files


def get_args():
    parser = argparse.ArgumentParser()
    parser.add_argument(
        "--source-file-name-match-type",
        dest="source_file_name_match_type",
        choices={"exact_match", "regex_match"},
        required=True,
    )
    parser.add_argument(
        "--source-folder-name", dest="source_folder_name", default="", required=False
    )
    parser.add_argument("--source-file-name", dest="source_file_name", required=True)
    parser.add_argument(
        "--destination-file-name",
        dest="destination_file_name",
        default="",
        required=False,
    )
    parser.add_argument(
        "--destination-folder-name",
        dest="destination_folder_name",
        default="",
        required=False,
    )
    parser.add_argument(
        "--service-account",
        dest="service_account",
        required=True,
    )
    parser.add_argument("--drive", dest="drive", default="", required=False)
    return parser.parse_args()


def main():
    args = get_args()
    dest_file_name = (
        args.destination_file_name if args.destination_file_name != "" else None
    )
    dest_folder_name = (
        args.destination_folder_name if args.destination_folder_name != "" else None
    )
    source_folder = args.source_folder_name if args.source_folder_name != "" else None

    drive = args.drive if args.drive != "" else None

    client = GoogleDriveClient(service_account=args.service_account)

    try:
        drive_id = (
            drive_utils.get_drive_id(drive_id=drive, service=client.service)
            if drive
            else None
        )
        folder_id = drive_utils.get_folder_id(
            folder_identifier=source_folder, service=client.service, drive_id=drive_id
        )

        # for downloading multiple file names
        if args.source_file_name_match_type == "regex_match":
            drive_files = drive_utils.get_file_matches(
                service=client.service,
                pattern=args.source_file_name,
                folder_id=folder_id,
                drive_id=drive_id,
            )

            client.logger.info(
                f"Found {len(drive_files)} files, preparing to download..."
            )
            for index, file in enumerate(drive_files):
                file_id = file["id"]
                file_name = file["name"]
                # rename the file appropriately
                dest_name = files.determine_destination_file_name(
                    source_full_path=file_name,
                    destination_file_name=dest_file_name,
                    file_number=index,
                )
                client.download(
                    file_id=file_id,
                    drive_file_name=file_name,
                    destination_file_name=dest_name,
                    destination_path=dest_folder_name,
                    drive=drive_id,
                    drive_folder=folder_id,
                )
                client.logger.info(f"Processed {dest_name}")
        # for single file downloads
        else:  # handles the case for exact_match, any other option will receive an argument error
            file_id = drive_utils.get_file_id(
                file_name=args.source_file_name,
                drive_id=drive_id,
                folder_id=folder_id,
                service=client.service,
            )
            if not file_id:
                client.logger.error(
                    f"File {args.source_file_name} not found or is not accessible to {client.email}. Ensure that the file exists in Google Drive and is shared with the service account"
                )
                sys.exit(client.EXIT_CODE_FILE_ACCESS_ERROR)

            client.download(
                file_id=file_id,
                drive_file_name=args.source_file_name,
                destination_file_name=dest_file_name,
                destination_path=dest_folder_name,
                drive=drive_id,
                drive_folder=folder_id,
            )

    except ExitCodeException as ec:
        client.logger.error(ec.message, sys.exit(ec.exit_code))

    except Exception as e:
        client.logger.error(
            f"Error in downloading the file from Google Drive due to {str(e)}"
        )
        sys.exit(client.EXIT_CODE_UNKNOWN_ERROR)

    else:
        client.logger.info("Successfully downloaded file(s) from Google Drive")


if __name__ == "__main__":
    main()



import os
import logging
import re
import glob
from typing import Optional, Union, List, Any, Set
from pathlib import Path

from shipyard_templates import ExitCodeException


def is_folder_shared(
    logger: logging.Logger, service_account_email: str, folder_id: str, drive_service
) -> bool:
    """Helper function to see if a provided folder is shared with the service account

    Args:
        drive_service (): The service connection
        service_account_email: The email of the service account
        folder_id: The ID of the folder in Google Drive

    Returns: True if folder is shared, False if not
    """
    try:
        permissions = drive_service.permissions().list(fileId=folder_id).execute()
        for permission in permissions.get("permissions", []):
            if (
                permission["type"] == "user"
                and permission["emailAddress"] == service_account_email
            ):
                return True

    except Exception as e:
        logger.warning(
            f"An exception was found during this call most likely indicating that no folder ID exists, returning False. Exception message: {str(e)}"
        )
        return False

    else:
        logger.warning("Folder ID is not shared with service account")
        return False


def does_file_exist(
    logger: logging.Logger,
    parent_folder_id: str,
    file_name: str,
    service,
    drive_id: Optional[str] = None,
) -> bool:
    """Helper function to see if the file already exists in the accessible Google Drive

    Args:
        service (): The drive connection
        parent_folder_id: The ID of the parent folder
        file_name: The name of the file
        drive_id: The optional ID of the shared drive

    Returns: True if exists, False if not

    """
    # query = f"name='{file_name}' and '{parent_folder_id}' in parents"
    query = f"'{parent_folder_id}' in parents and name='{file_name}'"
    try:
        if drive_id:
            results = (
                service.files()
                .list(
                    q=query,
                    includeItemsFromAllDrives=True,
                    corpora="drive",
                    driveId=drive_id,
                    supportsAllDrives=True,
                )
                .execute()
            )
        else:
            results = service.files().list(q=query).execute()
        if results.get("files", []) != []:
            return True
    except Exception as e:
        # this means that the file was not found
        logger.warning(
            f"An exception was thrown and now file was found, returning False: {str(e)}"
        )
        return False
    else:
        return False


def get_file_id(
    file_name: str,
    service,
    drive_id: Optional[str] = None,
    folder_id: Optional[str] = None,
) -> Union[str, None]:
    """Helper function to retrieve the file id in Google Drive

    Args:
        service (): The Google Drive service connection
        file_name: The name of the file to lookup in Google Drive
        drive_id: The Optional ID of the drive
        folder_id: The optional ID of the folder. This is only necessary if the file resides in a folder

    Raises:
        ExitCodeException:

    Returns: The ID of the file if exists, otherwise None

    """
    query = f"name='{file_name}'"
    if folder_id:
        query += f"and '{folder_id}' in parents"
    try:
        if drive_id:
            results = (
                service.files()
                .list(
                    q=query,
                    includeItemsFromAllDrives=True,
                    corpora="drive",
                    driveId=drive_id,
                    supportsAllDrives=True,
                )
                .execute()
            )
        else:
            results = service.files().list(q=query).execute()

    except Exception as e:
        raise ExitCodeException(f"Error in fetching file id: {str(e)}", exit_code=203)

    else:
        if "files" in results and len(results["files"]) > 0:
            return results["files"][0]["id"]
        return None


def create_remote_folder(
    folder_name: str,
    service,
    parent_id: Optional[str] = None,
    drive_id: Optional[str] = None,
) -> str:
    """Helper function to create a folder in Google Drive

    Args:
        service (): The google service connection
        folder_name: The name of the folder to create
        parent_id: The optional folder to place the newly created folder within
        drive_id: The optional drive to create the folder in

    Raises:
        ExitCodeException:

    Returns: The ID of the newly created folder

    """
    body = {"name": folder_name, "mimeType": "application/vnd.google-apps.folder"}
    if parent_id:
        body["parents"] = [parent_id]
    if drive_id and not parent_id:
        body["parents"] = [drive_id]

    try:
        folder = (
            service.files()
            .create(body=body, supportsAllDrives=True, fields=("id"))
            .execute()
        )
    except Exception as e:
        raise ExitCodeException(
            f"Failed to create folder {folder_name} in Goolge Drive", 208
        )
    return folder["id"]


def get_folder_id(
    service,
    folder_identifier: Optional[str] = None,
    drive_id: Optional[str] = None,
) -> Union[str, None]:
    """Helper function to grab the folder ID when provided either the name of the folder or the ID (preferred). This is instituted for backwards compatibility in the Shipyard blueprint

    Args:
        service (): The Google Drive service connection
        folder_identifier: The name of the folder or the ID from the URL

    Returns: The folder ID or None if nonexistent

    """
    if not folder_identifier:
        return None
    try:
        # every folder ID starts with 1 and is 33 chars long
        if is_folder_id(folder_identifier):
            return folder_identifier
        else:
            folder_names = folder_identifier.split("/")
            tmp_id = "root"  # this will be iteratively updated
            for folder_name in folder_names:
                if tmp_id == "root":
                    query = f"trashed=false and mimeType='application/vnd.google-apps.folder' and name='{folder_name}'"
                else:
                    query = f"'{tmp_id}' in parents and trashed=false and mimeType='application/vnd.google-apps.folder' and name='{folder_name}'"
                if not drive_id:
                    results = service.files().list(q=query).execute()
                else:
                    results = (
                        service.files()
                        .list(
                            # q=f"name = '{folder_identifier}' and mimeType = 'application/vnd.google-apps.folder'",
                            q=query,
                            fields="files(id)",
                            supportsAllDrives=True,
                            includeItemsFromAllDrives=True,
                            corpora="drive",
                            driveId=drive_id,
                        )
                        .execute()
                    )
                folders = results.get("files", [])
                if len(folders) > 1:
                    raise ExitCodeException(
                        f"Multiple folders with name {folder_identifier} found, please use the folder ID instead",
                        204,
                    )
                if folders:
                    tmp_id = folders[0]["id"]
                else:
                    return None
            return tmp_id

    except ExitCodeException as ec:
        raise ExitCodeException(ec.message, ec.exit_code)
    except Exception as e:
        return None


def get_drive_id(drive_id: str, service) -> Union[str, None]:
    """Helper function to grab the drive ID when either the name of the drive or the ID is provided. This is instituted for backwards compatibility in the Shipyard blueprint

    Args:
        service (): The Google Drive service connection
        drive_id:  The name of the drive or the ID from the URL

    Returns: The ID of the drive or None if not found

    """
    try:
        if len(drive_id) == 19 and str(drive_id).startswith("0A"):
            return drive_id
        else:
            results = (
                service.drives()
                .list(q=f"name = '{drive_id}'", fields="drives(id)")
                .execute()
            )
            drives = results.get("drives", [])
            if drives:
                return drives[0]["id"]
            else:
                return None
    except Exception:
        return None


def is_folder_id(folder_identifier: str) -> bool:
    """Helper function to determine if the input is a legitimate folder ID or a folder name

    Args:
        folder_identifier: Either the folder name or the ID from the URL

    Returns: True if the format matches that of a folder ID, false otherwise

    """
    if len(folder_identifier) == 33 and str(folder_identifier).startswith("1"):
        return True
    return False


def get_file_matches(
    service,
    pattern: str,
    folder_id: Optional[str] = None,
    drive_id: Optional[str] = None,
) -> List[Any]:
    """Helper function to return all the files that match a particular pattern

    Args:
        service (): The google service connection
        pattern: The pattern to search for
        folder_id: The folder to search within. If omitted, all file matches across all folders will be returned
        drive_id: The shared drive to search within

    Raises:
        ExitCodeException:

    Returns: The list of the matches

    """
    try:
        query = None
        if folder_id:
            query = f"'{folder_id}' in parents"
            if drive_id:
                if query:
                    results = (
                        service.files()
                        .list(
                            q=query,
                            supportsAllDrives=True,
                            includeItemsFromAllDrives=True,
                            corpora="drive",
                            driveId=drive_id,
                        )
                        .execute()
                    )
                else:
                    results = (
                        service.files()
                        .list(
                            supportsAllDrives=True,
                            includeItemsFromAllDrives=True,
                            corpora="drive",
                            driveId=drive_id,
                        )
                        .execute()
                    )
            else:
                if query:
                    results = service.files().list(q=query).execute()
                else:
                    results = service.files().list().execute()

            files = results.get("files", [])
        else:
            files = []
            all_folder_ids = get_all_folder_ids(service, drive_id=drive_id)
            for f_id in all_folder_ids:
                query = f"'{f_id}' in parents"
                if drive_id:
                    results = (
                        service.files()
                        .list(
                            q=query,
                            supportsAllDrives=True,
                            includeItemsFromAllDrives=True,
                            corpora="drive",
                            driveId=drive_id,
                        )
                        .execute()
                    )
                else:
                    results = service.files().list(q=query).execute()

                files.extend(results.get("files", []))

            # grab the files in the root
            if drive_id:
                root_results = (
                    service.files()
                    .list(
                        q="trashed=false and mimeType!='application/vnd.google-apps.folder'",
                        supportsAllDrives=True,
                        includeItemsFromAllDrives=True,
                        corpora="drive",
                        driveId=drive_id,
                    )
                    .execute()
                )
            else:
                root_results = (
                    service.files()
                    .list(
                        q="trashed=false and mimeType!='application/vnd.google-apps.folder'"
                    )
                    .execute()
                )

            files.extend(root_results.get("files", []))

        matches = []
        id_set = set()
        for f in files:
            if re.search(pattern, f["name"]) and f["id"] not in id_set:
                matches.append(f)
                id_set.add(f["id"])
    except Exception as e:
        raise ExitCodeException(f"Error in finding matching files: {str(e)}", 210)

    else:
        return matches


def get_all_folder_ids(service, drive_id: Optional[str] = None) -> List[Any]:
    # Set the query to retrieve all folders
    query = "mimeType='application/vnd.google-apps.folder' and trashed=false"

    # Execute the query to get the list of folders
    if drive_id:
        results = (
            service.files()
            .list(
                q=query,
                supportsAllDrives=True,
                includeItemsFromAllDrives=True,
                corpora="drive",
                driveId=drive_id,
            )
            .execute()
        )
    else:
        results = service.files().list(q=query).execute()

    folders = results.get("files", [])

    # Extract and return the folder IDs
    folder_ids = [folder["id"] for folder in folders]
    # folder_ids.append('root') # add so that the files not within a folder will be returned as well
    return folder_ids


def list_local_files(directory: Optional[str] = None) -> List[str]:
    """Returns all the files located within the directory path. If the dirname is not provided, then the current working directory will be used and it will span all subdirectories

    Args:
        dirname: The optional directory to span

    Returns: List of all the files

    """
    if directory is None:
        directory = os.getcwd()

    files = []
    for root, _, filenames in os.walk(directory):
        for filename in filenames:
            file_path = os.path.join(root, filename)
            files.append(file_path)

    return files



import os
import sys
import argparse
import re

from shipyard_templates import ExitCodeException
from shipyard_googledrive import GoogleDriveClient, drive_utils
from shipyard_bp_utils import files


def get_args():
    parser = argparse.ArgumentParser()
    parser.add_argument("--service-account", dest="service_account", required=True)
    parser.add_argument("--drive", required=False, default="")
    parser.add_argument("--source-file-name", dest="source_file_name", required=True)
    parser.add_argument(
        "--source-folder-name", dest="source_folder_name", required=False, default=""
    )
    parser.add_argument(
        "--source-file-name-match-type",
        dest="source_file_name_match_type",
        required=True,
        default="exact_match",
        choices={"exact_match", "regex_match"},
    )
    parser.add_argument(
        "--destination-file-name", dest="destination_file_name", required=False
    )
    parser.add_argument(
        "--destination-folder-name",
        dest="destination_folder_name",
        required=False,
        default="",
    )
    return parser.parse_args()


def main():
    args = get_args()
    client = GoogleDriveClient(
        service_account=args.service_account, shared_drive_name=args.drive
    )
    if args.source_folder_name != "":
        source_path = os.path.join(args.source_folder_name, args.source_file_name)
    else:
        source_path = args.source_file_name

    drive_folder = (
        args.destination_folder_name if args.destination_folder_name != "" else None
    )
    drive_file_name = (
        args.destination_file_name if args.destination_file_name != "" else None
    )
    drive_name = args.drive if args.drive != "" else None

    # for multiple file uploads

    if args.source_file_name_match_type == "regex_match":
        files_listed = drive_utils.list_local_files(
            args.source_folder_name if args.source_folder_name != "" else None
        )
        file_matches = files.find_all_file_matches(
            file_names=files_listed,
            file_name_re=re.compile(args.source_file_name),
        )
        if len(file_matches) == 0:
            client.logger.error(
                f"No files found matching regex {args.source_file_name}"
            )
            sys.exit(client.EXIT_CODE_FILE_NOT_FOUND)
        for index, file in enumerate(file_matches, start=1):
            new_file_name = files.determine_destination_file_name(
                source_full_path=file,
                destination_file_name=drive_file_name,
                file_number=index,
            )
            client.upload(
                file_path=file,
                drive_folder=drive_folder,
                drive=drive_name,
                drive_file_name=new_file_name,
            )
            client.logger.info(f"Processed {file}")

    # for single file uploads
    else:  # handles the case for exact_match, any other option will receive an argument error
        try:
            client.upload(
                file_path=source_path,
                drive_folder=drive_folder,
                drive=drive_name,
                drive_file_name=drive_file_name,
            )
        except ExitCodeException as ec:
            client.logger.error(ec.message)
            sys.exit(ec.exit_code)
        except Exception as e:
            client.logger.error("Error in uploading file to drive")
            client.logger.exception(str(e))
            sys.exit(client.EXIT_CODE_UPLOAD_ERROR)
        else:
            client.logger.info("Successfully loaded file(s) to Google Drive!")


if __name__ == "__main__":
    main()
